﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace SoundItOut
{
    internal static class MetaPhone
    {
        static Regex nonLetters = new Regex("[^A-Z]", RegexOptions.Compiled);
        static Regex vowels = new Regex("[AEIOU]", RegexOptions.Compiled);
        static Regex frontv = new Regex("[EIY]", RegexOptions.Compiled);
        static Regex varson = new Regex("[CSPTG]", RegexOptions.Compiled);

        internal static void GenerateMetaphoneKey2(string name, out string key)
        {
            key = string.Empty;
            StringBuilder name2 = new StringBuilder(nonLetters.Replace(name.ToUpper(), ""));
            StringBuilder key2 = new StringBuilder();

            // Exception cases

            string first2chars = string.Empty;
            if (name2.Length >= 2)
            {
                name2.ToString().Substring(0, 2);
            }
            switch (first2chars)
            {
                case "KN":
                case "GN":
                case "PN":
                case "AE":
                case "WR":
                    name2.Remove(0,1);
                    break;

                // Begin = WH, transform to W
                case "WH":
                    name2.Remove(1,1);
                    break;
            }

            // Begin = X, transform to S
            if (name2[0] == 'X')
            {
                name2[0] = 'S';
            }

            int length = name2.Length;

            for (int i = 0; i < length; i++)
            {
                char c = name2[i];

                // From original BASIC implementation, it uses 4 key max limit .. not sure why
                // Not limiting metaphone to 4 chars at this moment
                //while (key.Length < 4)
                //{
                //}

                if (c != 'C' && i > 0 && name2[i - 1] == c)
                {
                    continue;
                }

                if (vowels.IsMatch(c.ToString()) && i == 0)
                {
                    key2.Append(c);
                    continue;
                }

                switch (c)
                {
                    case 'B':
                        if (i == length - 1 && name2[length - 2] == 'M')
                        {
                            continue;
                        }
                        key2.Append(c);
                        break;

                    case 'C':
                        if (!(i > 0 && name2[i - 1] == 'S' && i + 1 < length && frontv.IsMatch(name2[i + 1].ToString())))
                        {
                            if (i + 2 < length && name2.ToString().Substring(i, 3) == "CIA")
                            {
                                key2.Append("X");
                            }

                            else if (i + 1 < length && frontv.IsMatch(name2[i + 1].ToString()))
                            {
                                key2.Append("S");
                            }

                            else if (i > 0 && i + 1 < length && name2[i - 1] == 'S' && name2[i + 1] == 'H')
                            {
                                key2.Append("K");
                            }

                            else if (i + 1 < length && name2[i + 1] == 'H')
                            {
                                // BUGFIX: 
                                // IF N = 1 AND (N + 2) <= L AND INDEX(VOWELS,ENAME[N + 2,1],1) = 0 THEN
                                // means when it does not find a vowel after H. I missed the 'not'
                                if (i == 0 && i + 2 < length && !vowels.IsMatch(name2[i + 2].ToString()))
                                {
                                    key2.Append("K");
                                }

                                else
                                {
                                    key2.Append("X");
                                }
                            }

                            else
                            {
                                key2.Append("K");
                            }
                        }
                        break;

                    case 'D':
                        if (i + 2 < length && name2[i + 1] == 'G' && frontv.IsMatch(name2[i + 2].ToString()))
                        {
                            key2.Append("J");
                        }

                        else
                        {
                            key2.Append("T");
                        }

                        break;

                    case 'G':
                        if (i + 2 < length && name2[i + 1] == 'H' && !vowels.IsMatch(name2[i + 2].ToString()))
                        {
                            continue;
                        }

                        if (
                            i + 1 < length && name2[i + 1] == 'N' || 
                            (i + 3 < length && name2[i+1] == 'N' && name2[i+2] == 'E' && name2[i+3] == 'D'))
                        {
                            continue;
                        }

                        if (i > 0 && i + 1 < length && name2[i -1] == 'D' && frontv.IsMatch(name2[i + 1].ToString()))
                        {
                            continue;
                        }

                        if (i > 0 && name2[i - 1] == 'G')
                        {
                            continue;
                        }

                        if (i + 1 < length && frontv.IsMatch(name2[i + 1].ToString()))
                        {
                            key2.Append("J");
                        }

                        else
                        {
                            key2.Append("K");
                        }

                        break;

                    case 'H':
                        if (!(i + 1 == length || (i > 0 && varson.IsMatch(name2[i - 1].ToString()))))
                        {
                            if (vowels.IsMatch(name2[i + 1].ToString()))
                            {
                                key2.Append("H");
                            }
                        }

                        break;

                    case 'F':
                    case 'J':
                    case 'L':
                    case 'M':
                    case 'N':
                    case 'R':
                        key2.Append(c);
                        break;

                    case 'K':
                        if (i > 0 && name2[i - 1] != 'C')
                        {
                            key2.Append("K");
                        }
                        else if (i == 0)
                        {
                            key2.Append("K");
                        }
                        break;

                    case 'P':
                        if (i + 1 < length && name2[i + 1] == 'H')
                        {
                            key2.Append("F");
                        }
                        else
                        {
                            key2.Append("P");
                        }
                        break;

                    case 'Q':
                        key2.Append("K");
                        break;

                    case 'S':
                        if (i > 0 && i + 2 < length && name2[i + 1] == 'I' && (name2[i + 2] == 'O' || name2[i + 2] == 'A'))
                        {
                            key2.Append("X");
                        }
                        else if (i + 1 < length && name2[i + 1] == 'H')
                        {
                            key2.Append("X");
                        }
                        else
                        {
                            key2.Append("S");
                        }

                        break;

                    case 'T':
                        if (i > 0 && i + 2 < length && name2[i + 1] == 'I' && (name2[i + 2] == 'O' && name2[i + 2] == 'A'))
                        {
                            key2.Append("X");
                        }
                        else if (i + 1 < length && name2[i + 1] == 'H')
                        {
                            if (!(i > 0 && name2[i - 1] == 'T'))
                            {
                                key2.Append("0");
                            }
                        }
                        else if (i + 2 < length && name2[i + 1] == 'C' && name2[i + 2] == 'H')
                        {
                            continue;
                        }
                        else
                        {
                            key2.Append("T");
                        }

                        break;

                    case 'V':
                        key2.Append("F");
                        break;

                    case 'W':
                    case 'Y':
                        if (i + 1 < length && vowels.IsMatch(name2[i + 1].ToString()))
                        {
                            key2.Append(c);
                        }
                        break;

                    case 'X':
                        key2.Append("KS");
                        break;

                    case 'Z':
                        key2.Append("S");
                        break;
                }
            }

            key = key2.ToString();
        }
    }
}
