﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace SoundItOut
{
    internal static class SoundEx
    {
            //1 -The first letter of the name is the letter of the Soundex code, and is not coded to a number.
            //Replace consonants with digits as follows (after the first letter):
            //2 b, f, p, v => 1
            //3 c, g, j, k, q, s, x, z => 2
            //4 d, t => 3
            //5 l => 4
            //6 m, n => 5
            //7 r => 6
            //8 vowels, h, w are not coded
            //9 Two adjacent letters with the same number are coded as a single number. Letters with the same number separated by an h or w are also coded as a single number.
            //10 Continue until you have one letter and three numbers. If you run out of letters, fill in 0s until there are three numbers.

        internal static void GenerateSoundexKey(string name, out string key)
        {
            key = string.Empty;
            string upperName = name.ToUpper();

            //Let's strip non A-Z characters
            upperName = Regex.Replace(upperName, "[^A-Z]", string.Empty, RegexOptions.Compiled);

            //step 1
            string fLetter = upperName.Substring(0, 1);

            //Check if it starts with a vowel, H, or Y
            bool startsVowel = SIOHelpers.VOWELS.IsMatch(fLetter) || upperName.StartsWith("H") || upperName.StartsWith("Y");

            //2010-12-04 (billyd) - turns out wikipedia is wrong when it lists the below.
            //WIKI: 1 -The first letter of the name is the letter of the Soundex code, and is not coded to a number.
            //it actually IS coded, just not added to the key. and this affects step 9 
            //(so if you have Pfister, the P is added as P and the f is then thrown out 
            //      because it and P both represent 1; though no 1 will be added to the key, because P is)
      
            //upperName = upperName.Length > 1 ? upperName.Substring(1) : string.Empty;


            upperName = _step_one.Replace(upperName, "1");
            upperName = _step_two.Replace(upperName, "2");
            upperName = upperName.Replace("D", "3").Replace("T", "3"); 
            upperName = upperName.Replace("L", "4");
            upperName = upperName.Replace("M", "5").Replace("N", "5");
            upperName = upperName.Replace("R", "6");

            //step8
            upperName = upperName.Replace("H", string.Empty).Replace("W", string.Empty);

            //step 9
            upperName = SIOHelpers.CollapseAdjacentRepeating(upperName);


            //Doing this after step 9 so we don't screw up step 9 
            //(original algorithm looped through original rather than using replace)
            //  because we're using replace, the drop non h and w values move after step 9
            //step 8 - drop vowels, h, 2, and y
            upperName = SIOHelpers.VOWELS.Replace(upperName.Replace("Y", string.Empty), string.Empty);


            if (startsVowel)
            {
                key = fLetter + upperName;
            }
            else
            {
                //2010-12-04 (billyd) - as explained above, for step 1. We coded the first letter, now we should remove it
                key = fLetter + (upperName.Length > 1 ? upperName.Substring(1) : string.Empty);
            }

            //If you run out of letters, fill in 0s until there are three numbers.
            while (key.Length < 4)
            {
                key += "0";
            }

            //last minute length check
            if (key.Length > 4)
            {
                key = key.Substring(0, 4);
            }
        }

        static Regex _vowels_extended = new Regex("[AEIOUWHY]", RegexOptions.Compiled);
        static Regex _step_one = new Regex("[BFPV]", RegexOptions.Compiled);
        static Regex _step_two = new Regex("[CGJKQSXZ]", RegexOptions.Compiled);
    }
}
